#include <Servo.h>

#define pump  2
#define valve 4
#define ROWS 3 
#define COLS 3 

const int numRows = 4;       // numero di righe tastierino
const int numCols = 3;       // numero di colonne tastierino
const int debounceTime = 20; // millisecondi necessari allo switch per diventare stabile

// Mappa dei caratteri sul tastierino
const int keymap[numRows][numCols] = 
{
  { 6, 3, 0  } ,
  { 7, 4, 1  } ,
  { 8, 5, 2  } ,
  { 11, 9, 10  }
};

Servo base;
Servo shoulder;
Servo elbow;

int PosTris[9][3]; //Posizioni sul tris, 9 caselle, 3 servo da comandare
int RecuperoO[5][3]; //Posizioni per prendere le pedine O
int PausaO[3]; 
int RecuperoX[5][3]; //Posizioni per prendere le pedine X
int PausaX[3];
int PosBase[3]; //Posizioni base
int rowPins[numRows] = {3, 15, 16, 17}; // Pin di arduino connessi ai pin 1,2,3 e 4 delle righe del keypad
int colPins[numCols] = {18, 19, 5};   // Pin di arduino connessi ai pin 5,6,7 delle colonne del keypad

//Questi servono per il movimento
int step_base;
int step_shoulder;
int step_elbow;


int angolo, mossa, mossaDef, turno, i, j, an, ce, cr, n;

//Posizioni delle pedine nella scacchiera 
int pos[3][3];  /*posizione reale pedine*/
int pos1[3][3]; /*posizione fittizia dopo il ruota*/

int numOprese; //numero di pedine prese per O
int numXprese; //numero di pedine prese per X

bool vittoria, ruotato1, ruotato2;
bool verificaTris, bloccaterzamossa, bloccaMossaRandom, bloccaMovimento; 
bool nuovaPartita, terminaPartita, riordinaPedine; 

void setup() {
  Serial.begin(9600);
  randomSeed(analogRead(0));
  
  PosBase[0] = 110;
  PosBase[1] = 120;
  PosBase[2] = 75;

  step_base = PosBase[0];
  step_shoulder = PosBase[1];
  step_elbow = PosBase[2];

  //Tutte le posizioni delle 9 caselle del tris per ciascun servo
  PosTris[0][0] = 95;
  PosTris[0][1] = 64;
  PosTris[0][2] = 133;
  
  PosTris[1][0] = 90;
  PosTris[1][1] = 82;
  PosTris[1][2] = 147;
  
  PosTris[2][0] = 80;
  PosTris[2][1] = 100;
  PosTris[2][2] = 153;
  
  PosTris[3][0] = 115;
  PosTris[3][1] = 69;
  PosTris[3][2] = 138;
  
  PosTris[4][0] = 115;
  PosTris[4][1] = 84;
  PosTris[4][2] = 149;
  
  PosTris[5][0] = 115;
  PosTris[5][1] = 104;
  PosTris[5][2] = 153;
  
  PosTris[6][0] = 129;
  PosTris[6][1] = 64;
  PosTris[6][2] = 134;
  
  PosTris[7][0] = 135;
  PosTris[7][1] = 81;
  PosTris[7][2] = 146;
  
  PosTris[8][0] = 142;
  PosTris[8][1] = 99;
  PosTris[8][2] = 153; 

  //Posizioni da cui recupero le pedine O
  RecuperoO[0][0] = 141;
  RecuperoO[0][1] = 51;
  RecuperoO[0][2] = 123;
  
  RecuperoO[1][0] = 146;
  RecuperoO[1][1] = 64;
  RecuperoO[1][2] = 135;
  
  RecuperoO[2][0] = 151;
  RecuperoO[2][1] = 74;
  RecuperoO[2][2] = 144;
  
  RecuperoO[3][0] = 161;
  RecuperoO[3][1] = 83;
  RecuperoO[3][2] = 149;
  
  RecuperoO[4][0] = 169;
  RecuperoO[4][1] = 92;
  RecuperoO[4][2] = 153;
  
  PausaO[0] = 102;
  PausaO[1] = 108;
  PausaO[2] = 88;

  //Posizioni da cui recupero le pedine X
  RecuperoX[0][0] = 81;
  RecuperoX[0][1] = 51;
  RecuperoX[0][2] = 123;
  
  RecuperoX[1][0] = 76;
  RecuperoX[1][1] = 64;
  RecuperoX[1][2] = 136;
  
  RecuperoX[2][0] = 70;
  RecuperoX[2][1] = 74;
  RecuperoX[2][2] = 142;
  
  RecuperoX[3][0] = 62;
  RecuperoX[3][1] = 82;
  RecuperoX[3][2] = 146;
  
  RecuperoX[4][0] = 48;
  RecuperoX[4][1] = 93;
  RecuperoX[4][2] = 153;

  PausaX[0] = 102;
  PausaX[1] = 108;
  PausaX[2] = 88;

  /*Inizializzo la scacchiera a 1 per ciascuna casella*/
  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++)
      pos[i][j] = 1;

  base.attach(11);
  shoulder.attach(10);
  elbow.attach(9);
  
  vittoria = false;
  verificaTris = false;
  ruotato1 = false;
  ruotato2 = false;
  verificaTris = false;
  bloccaterzamossa = false;
  bloccaMossaRandom = false;
  bloccaMovimento = false;
  nuovaPartita = false;
  terminaPartita = false;
  riordinaPedine = false;

  turno = random(2); 
  mossa = 0;
  mossaDef = 0;
  an = 0;
  ce = 0;
  cr = 0;
  numOprese = 0;
  numXprese = 0;

  //Tastierino
  for(int row = 0; row < numRows; row++){
    pinMode(rowPins[row],INPUT);       // Imposta i p1n delle righe come input
    digitalWrite(rowPins[row],HIGH);   // Imposta le righe a HIGH (inattiva) - abilita le resistenze di pull-ups
  }
  for(int column = 0; column < numCols; column++){
    pinMode(colPins[column],OUTPUT);     
    digitalWrite(colPins[column],HIGH);  
  }
  
  //Pompetta
  pinMode(pump, OUTPUT);
  pinMode(valve, OUTPUT);
  digitalWrite(pump, HIGH);
  digitalWrite(valve, HIGH); //HIGH SUCCHIO  LOW RILASCIO

  //Posizione iniziale
  base.write(PosBase[0]);
  shoulder.write(PosBase[1]);
  elbow.write(PosBase[2]);

  delay(2000);
}

void loop() 
{
  if(vittoria == false){
    if(turno == 0){ /*robot attacca*/
      bloccaMossaRandom = false;
      if(mossa == 0)
        primaMossa(&bloccaMossaRandom, &ruotato2, &n, pos, pos1, &an, &ce, &cr);

      Ruota(&ruotato1, &angolo, &n, pos, pos1, ruotato2, turno);
      
      chiudiMioTris(pos1, &n, &verificaTris, &bloccaterzamossa, &bloccaMossaRandom, &vittoria);
      
      if(verificaTris == false)
          chiudiSuoTris(pos1, &n, &bloccaterzamossa, &bloccaMossaRandom);
      
      if(mossa == 1)
        secondaMossa(pos1, &n, &ce, &an, &cr, &bloccaMossaRandom);

      if(bloccaterzamossa == false && mossa == 2)
          terzaMossa(pos1, &n, ce, an, cr, &bloccaMossaRandom);

      if(bloccaMossaRandom == false)
          mossaRandom(pos1, &n);
   
      mossa++;
          
      riruota(&angolo, &n, pos, pos1);  
      
      Posiziona(n);

      controllavittoria(pos1, &vittoria);
      controllapareggio(pos1, &vittoria, &bloccaMovimento);

      if(vittoria == false){
        input(pos, &vittoria);
        if(terminaPartita == true){
          input(pos, &vittoria);
          if(riordinaPedine == true){
            sistemaTutto();
            input(pos, &vittoria);
            if(nuovaPartita == true){
                servoMovementdown(40, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
                for(i = 0; i < 3; i++)
                  for(j = 0; j < 3; j++)
                    pos[i][j] = 1;
        
                mossa = 0;
                mossaDef = 0;
                an = 0;
                ce = 0;
                cr = 0;
                numOprese = 0;
                numXprese = 0;
                verificaTris = false;
                ruotato1 = false;
                ruotato2 = false;
                verificaTris = false;
                bloccaterzamossa = false;
                bloccaMossaRandom = false;
                bloccaMovimento = false;
                terminaPartita = false;
                riordinaPedine = false;
                nuovaPartita = false;
                turno = abs(turno - 1);
            }            
          }
        }
      } 
    }else if(turno == 1){ //Robot difende
        bloccaMovimento = false;
        bloccaMossaRandom = false;
        
        input(pos, &vittoria);
        if(terminaPartita == true){
          input(pos, &vittoria);
          if(riordinaPedine == true){
            sistemaTutto();
            input(pos, &vittoria);
            if(nuovaPartita == true){
                servoMovementdown(40, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
                for(i = 0; i < 3; i++)
                  for(j = 0; j < 3; j++)
                    pos[i][j] = 1;
        
                mossa = 0;
                mossaDef = 0;
                an = 0;
                ce = 0;
                cr = 0;
                numOprese = 0;
                numXprese = 0;
                verificaTris = false;
                ruotato1 = false;
                ruotato2 = false;
                verificaTris = false;
                bloccaterzamossa = false;
                bloccaMossaRandom = false;
                bloccaMovimento = false;
                terminaPartita = false;
                riordinaPedine = false;
                nuovaPartita = false;
                turno = abs(turno - 1);
            }            
          }
        }else{
          Ruota(&ruotato1, &angolo, &n, pos, pos1, ruotato2, turno);
        
          chiudiMioTris(pos1, &n, &verificaTris, &bloccaterzamossa, &bloccaMossaRandom, &vittoria);
          
          if(verificaTris == false)
              chiudiSuoTris(pos1, &n, &bloccaterzamossa, &bloccaMossaRandom);
          
          if(mossaDef == 0)
            primaMossaDef(pos1, &n, &bloccaMossaRandom);
          
          if(bloccaterzamossa == false)
            secondaMossaDef(pos1, &n, mossaDef, &bloccaMossaRandom);
  
          if(bloccaMossaRandom == false)
              mossaRandom(pos1, &n);
          
          mossaDef++;
       
          riruota(&angolo, &n, pos, pos1);  
  
          controllavittoria(pos1, &vittoria);
          controllapareggio(pos1, &vittoria, &bloccaMovimento);
  
          if(bloccaMovimento==false)
            Posiziona(n);
      }                  
    }
  }
  if(vittoria == true){
    servoMovementdown(40, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
    input(pos, &vittoria);
    if(riordinaPedine == true){
       sistemaTutto();
       input(pos, &vittoria);
       if(nuovaPartita == true){
          servoMovementdown(40, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
          for(i = 0; i < 3; i++)
            for(j = 0; j < 3; j++)
              pos[i][j] = 1;
    
          mossa = 0;
          mossaDef = 0;
          an = 0;
          ce = 0;
          cr = 0;
          numOprese = 0;
          numXprese = 0;
          verificaTris = false;
          ruotato1 = false;
          ruotato2 = false;
          verificaTris = false;
          bloccaterzamossa = false;
          bloccaMossaRandom = false;
          bloccaMovimento = false;
          terminaPartita = false;
          riordinaPedine = false;
          nuovaPartita = false;
          vittoria = false;
          turno = abs(turno - 1);
       }            
     }
  }  
}

void servoMovementup(int stepDelay, int vBase, int vShoulder, int vElbow, int vWrist_ver, int vWrist_rot, int vgripper) {

  stepDelay = 20;
  int exit = 1;

  //Until the all motors are in the desired position
  while (exit)
  {
    //For each servo motor if next degree is not the same of the previuos than do the movement
    while (vBase != step_base)
    {
      base.write(step_base);
      //One step ahead
      if (vBase > step_base) {
        step_base++;
      }
      //One step beyond
      if (vBase < step_base) {
        step_base--;
      }
      delay(stepDelay);
    }

    while (vShoulder != step_shoulder)
    {
      shoulder.write(step_shoulder);
      //One step ahead
      if (vShoulder > step_shoulder) {
        step_shoulder++;
      }
      //One step beyond
      if (vShoulder < step_shoulder) {
        step_shoulder--;
      }
      delay(stepDelay);
    }

    while (vElbow != step_elbow)
    {
      elbow.write(step_elbow);
      //One step ahead
      if (vElbow > step_elbow) {
        step_elbow++;
      }
      //One step beyond
      if (vElbow < step_elbow) {
        step_elbow--;
      }
      delay(stepDelay);
    }
    //delay between each movement
    

    //It checks if all the servo motors are in the desired position
    if ((vBase == step_base) && (vShoulder == step_shoulder)
        && (vElbow == step_elbow)) {
      /*step_base = vBase;
      step_shoulder = vShoulder;
      step_elbow = vElbow;*/
      exit = 0;
    } else {
      exit = 1;
    }
  }
}

void servoMovementdown(int stepDelay, int vBase, int vShoulder, int vElbow, int vWrist_ver, int vWrist_rot, int vgripper) {

  stepDelay = 20;
  int exit = 1;

  //Until the all motors are in the desired position
  while (exit)
  {
    //For each servo motor if next degree is not the same of the previuos than do the movement
    while (vElbow != step_elbow)
    {
      elbow.write(step_elbow);
      //One step ahead
      if (vElbow > step_elbow) {
        step_elbow++;
      }
      //One step beyond
      if (vElbow < step_elbow) {
        step_elbow--;
      }
      delay(stepDelay);
    }
    //delay between each movement

    while (vShoulder != step_shoulder)
    {
      shoulder.write(step_shoulder);
      //One step ahead
      if (vShoulder > step_shoulder) {
        step_shoulder++;
      }
      //One step beyond
      if (vShoulder < step_shoulder) {
        step_shoulder--;
      }
      delay(stepDelay);
    }
    
    while (vBase != step_base)
    {
      base.write(step_base);
      //One step ahead
      if (vBase > step_base) {
        step_base++;
      }
      //One step beyond
      if (vBase < step_base) {
        step_base--;
      }
      delay(stepDelay);
    }
    
    //It checks if all the servo motors are in the desired position
    if ((vBase == step_base) && (vShoulder == step_shoulder)
        && (vElbow == step_elbow)) {
      /*step_base = vBase;
      step_shoulder = vShoulder;
      step_elbow = vElbow;*/
      exit = 0;
    } else {
      exit = 1;
    }
  }
}

void Posiziona(int n) //Il robot ha deciso di metterla in n e ora la mette
{
  //prendo la pedina e mi alzo pronto per posizionarla
  servoMovementup(20, RecuperoX[numXprese][0], RecuperoX[numXprese][1], RecuperoX[numXprese][2], 100, 90, 90);
  delay(500);
  digitalWrite(valve, HIGH);
  digitalWrite(pump, LOW);
  delay(1000);
  servoMovementdown(20, PausaX[0], PausaX[1], PausaX[2], 100, 90, 90);
  
  //La posiziono in posizione n e torno in base
  servoMovementup(20, PosTris[n][0], PosTris[n][1], PosTris[n][2], 100, 90, 90);
  delay(1000);
  digitalWrite(valve, LOW);
  digitalWrite(pump, HIGH);
  delay(500);
  servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
  
  numXprese++;
}

void input(int pos[][COLS], bool * vittoria) /*dico dove voglio mettere la mia pedina e il robot la mette*/
{
  int n, n1;
  int premuto = 0;  
  while(premuto == 0){                             
    for(int column = 0; column < numCols && premuto == 0; column++){
      digitalWrite(colPins[column],LOW);         // Attiva la colonna corrente per la lettura.
      for(int row = 0; row < numRows && premuto == 0; row++){     // Scorro tutte le righe cercando un tasto premuto
        if(digitalRead(rowPins[row]) == LOW){     // E' premuto il tasto
          premuto = 1;
          delay(debounceTime);                     // attendo che il segnale si stabilizzi 
          while(digitalRead(rowPins[row]) == LOW); // attendo che il tasto sia rilasciato
          n1 = keymap[row][column]; 
          if(n1 != 9 && n1 != 10 && n1 != 11 && * vittoria == false && terminaPartita == false && riordinaPedine == false && nuovaPartita == false){
            //prendo la pedina e mi alzo pronto per posizionarla
            n = n1;
            servoMovementup(20, RecuperoO[numOprese][0], RecuperoO[numOprese][1], RecuperoO[numOprese][2], 100, 90, 90);
            delay(500);
            digitalWrite(valve, HIGH);
            digitalWrite(pump, LOW);
            delay(1000);
            servoMovementdown(20, PausaO[0], PausaO[1], PausaO[2], 100, 90, 90);
            
            //La posiziono in posizione n e torno in base
            servoMovementup(20, PosTris[n][0], PosTris[n][1], PosTris[n][2], 100, 90, 90);
            delay(1000);
            digitalWrite(valve, LOW);
            digitalWrite(pump, HIGH);
            delay(500);
            servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);
            
            numOprese++;
            if(n == 0)
              pos[0][0] = 5;
            else{
              pos[n1/3][n1%3] = 5;
            }
          }else if(n1 == 9 && riordinaPedine == true){
            nuovaPartita = true;
          }else if(n1 == 10 && * vittoria == false){
            terminaPartita = true;
          }else if(n1 == 11 && (terminaPartita == true || * vittoria == true)){
            riordinaPedine = true;
          }else{
            premuto = 0;
          }
        }
      }
      digitalWrite(colPins[column],HIGH);     // Rendo inattiva la colonna.
    }
  }
}
//turno e vittoria = false

void sistemaTutto()
{
   int i, j;
   numXprese = 0;
   numOprese = 0;
   for(i = 0; i < 3; i++){
    for(j = 0; j < 3; j++){
      if(pos[i][j] == 3){ //X
          n = (3 * i) + j;
          servoMovementup(20, PosTris[n][0], PosTris[n][1], PosTris[n][2], 100, 90, 90);
          delay(1000);
          digitalWrite(valve, HIGH);
          digitalWrite(pump, LOW);
          delay(1000);
          servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);

          servoMovementup(20, RecuperoX[numXprese][0], RecuperoX[numXprese][1], RecuperoX[numXprese][2], 100, 90, 90);
          delay(1000);
          digitalWrite(valve, LOW);
          digitalWrite(pump, HIGH);
          delay(500);
          servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);

          numXprese++;
      }else if(pos[i][j] == 5){  //O
          n = (3 * i) + j;
          servoMovementup(20, PosTris[n][0], PosTris[n][1], PosTris[n][2], 100, 90, 90);
          delay(1000);
          digitalWrite(valve, HIGH);
          digitalWrite(pump, LOW);
          delay(1000);
          servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);

          servoMovementup(20, RecuperoO[numOprese][0], RecuperoO[numOprese][1], RecuperoO[numOprese][2], 100, 90, 90);
          delay(1000);
          digitalWrite(valve, LOW);
          digitalWrite(pump, HIGH);
          delay(500);
          servoMovementdown(20, PosBase[0], PosBase[1], PosBase[2], 100, 90, 90);

          numOprese++;
      }
    }
   }  
}


void primaMossa(bool * bloccaMossaRandom, bool * ruotato2, int * n, int pos[][COLS], int pos1[][COLS], int * an, int * ce, int * cr)
{
    int x, u, i, j;
    
    x = random(10);
    if(x < 5){ /*Lo metto in centro*/
      * n = 4;
      pos[1][1] = 3;
      * ce = 1;
      for(i=0;i<3;i++)
        for(j=0;j<3;j++)
         pos1[i][j]=pos[i][j];

      * ruotato2 = true;
    }
    else if(x >= 5 && x < 9) /*Lo metto in angoli*/
    {
        u = random(4);
        if(u == 0){
          * n = 0;
          pos[0][0] = 3;
        }else if(u == 1){
          * n = 2;
          pos[0][2] = 3;
        }else if(u == 2){
          * n = 6;
          pos[2][0] = 3;
        }else if(u == 3){
          * n = 8;
          pos[2][2] = 3;
        }
        * an = 1;
    }
    else /*lo metto in croce*/
    {
        u = random(4);
        if(u == 0){
          * n = 1;
           pos[0][1] = 3;
        }else if(u == 1){
          * n = 3;
          pos[1][0] = 3;
        }else if(u == 2){
          * n = 5;
           pos[1][2] = 3;
        }else if(u == 3){
          * n = 7;
          pos[2][1] = 3;
        }
        * cr = 1;
    }
    * bloccaMossaRandom = true;
}

void secondaMossaConPrimaAlCentro(int pos1[][COLS], int * n)
{
  if(pos1[1][1]==3){
    if(pos1[0][1]==5){
      * n = 0;
      pos1[0][0] = 3;
    }else if(pos1[0][0]==5){
      * n = 8;
      pos1[2][2] = 3;
    }
  }
}

void secondaMossaConPrimaInAngolo(int pos1[][COLS], int * n)
{
  int i, j, h;

  if(pos1[0][0]==3)
    for(i=0;i<3;i++)
      for(j=0;j<3;j++)
        if(pos1[i][j]==5){
          h = 3 * i + j;   
          if(h == 1 || h == 5 || h == 3 || h == 7){
            pos1[1][1] = 3;
            * n = 4;
          }else if(h == 2 || h == 4 || h == 6){
            pos1[2][2] = 3;
            * n = 8;
          }else if(h == 8){
            pos1[2][0] = 3;
            * n = 6;
          } 
        }
}

void secondaMossaConPrimaInCroce(int pos1[][COLS], int * n)
{
  int i, j, h;

  if(pos1[0][1]==3){
    for(i=0;i<3;i++){
      for(j=0;j<3;j++){
        if(pos1[i][j]==5){
          h= 3 * i + j;
          switch(h){
            case 0:
              * n=8;
              pos1[2][2]=3;
              break;
            case 2:
              * n=6;
              pos1[2][0]=3;
              break;
            case 4:
              * n=5;
              pos1[1][2]=3;
              break;
            case 5:
              * n=4;
              pos1[1][1]=3;
              break;
            case 8:
              * n=2;
              pos1[0][2]=3;
              break;
            case 3:
              * n=4;
              pos1[1][1]=3;
              break;
            case 6:
              * n=0;
              pos1[0][0]=3;
              break;
            case 7:
              * n=2;
              pos1[0][2]=3;
              break;
          }    
        }
      }
    }
  }
}

void secondaMossa(int pos1[][COLS], int * n, int * ce, int * an, int * cr, bool * bloccaMossaRandom)
{ 
    int n1;

    n1 = * n;
  if(* ce==1)
  {
    secondaMossaConPrimaAlCentro(pos1, &n1); 
    * bloccaMossaRandom=true;  
    * ce = 2;
  }
  if(* an==1)
  {
    secondaMossaConPrimaInAngolo(pos1, &n1);
    * bloccaMossaRandom=true;
    * an = 2;
  }
  if(* cr==1)
  {
    secondaMossaConPrimaInCroce(pos1, &n1); 
    * bloccaMossaRandom=true;
    * cr = 2;
  }    
    * n = n1;
}

void terzaMossaConPrimaAlCentro(int pos1[][COLS], int * n)
{
  int i, j, stopp;
  stopp = 0;

  for(i=0; i<3 && stopp == 0; i++){
    for(j=0; j<3 && stopp == 0; j++){
      if(pos1[i][j] == 5){
        if(i == 1){
          * n = 2;
          pos1[0][2] = 3;
          stopp = 1;
        }
        else if(j == 1){
          * n = 6;
          pos1[2][0]=3;
          stopp = 1;
        }
      }     
    }
  } 
}

void terzaMossaConPrimaInAngolo(int pos1[][COLS], int * n)
{
  if(pos1[0][0]==3){
    if(pos1[1][1]==3 && pos1[2][2]==5){
      if(pos1[0][1]==5){
        * n = 6;
        pos1[2][0]=3;
      }
      else if(pos1[1][0]==5){
        * n = 2;
        pos1[0][2]=3;
      }
    }
    else if(pos1[2][2]==5 && pos1[2][0]==3 && pos1[1][0]==5){
      * n = 2;
      pos1[0][2]=3;
    }
  }
}

void terzaMossaConPrimaInCroce(int pos1[][COLS], int * n)
{
  int n1;
  if(pos1[0][1]==3){
    if((pos1[0][0]==5 && pos1[2][2]==3) && (pos1[0][2]==5 || pos1[1][2]==5)){
        * n = 7;
        pos1[2][1]=3;
    }else if(pos1[0][2]==5 && pos1[2][0]==3 && (pos1[0][0]==5 || pos1[1][0]==5)){
        * n = 7;
        pos1[2][1]=3;
    }else if(pos1[1][1]==3 && pos1[2][1]==5 && pos1[1][0]==5){
        * n = 2;
        pos1[0][2]=3;
    }else if(pos1[1][1]==3 && pos1[2][1]==5 && pos1[1][2]==5){
        * n = 0;
        pos1[0][0]=3;
    }else if(pos1[1][1]==5 && pos1[1][2]==3 && (pos1[1][0]==5 || pos1[2][1]==5)){
        * n = 2;
        pos1[0][2]=3;
    }else if(pos1[2][1]==5 && pos1[0][2]==3 && pos1[0][0]==5){
      * n = 6;
      pos1[2][0]=3;       
    }else{
      mossaRandom(pos1, &n1);
      * n = n1;
    }
  }
}

void terzaMossa(int pos1[][COLS], int * n, int ce, int an, int cr, bool * bloccaMossaRandom)
{
  int n1;

  n1 = * n;
  if(ce == 2)
    terzaMossaConPrimaAlCentro(pos1, &n1);
  else if(an == 2)
    terzaMossaConPrimaInAngolo(pos1, &n1);
  else if(cr == 2)
    terzaMossaConPrimaInCroce(pos1, &n1);
  * bloccaMossaRandom = true;
  * n = n1;
}

void mossaRandom(int pos1[][COLS], int * n)
{
  int i, j, stopp;

  stopp = 0;
  for(i=0; i<3 && stopp == 0; i++)
    for(j=0;j<3 && stopp == 0;j++)
      if(pos1[i][j]==1){
        pos1[i][j]=3;
        * n = 3 * i + j;
        stopp = 1;
      }      
}

void primaMossaConSuaAlCentro(int pos1[][COLS], int * n)
{
    * n = 0;
    pos1[0][0] = 3;
}

void primaMossaConSuaInAngolo(int pos1[][COLS], int * n)
{
    * n = 4;
    pos1[1][1] = 3; 
}

void primaMossaConSuaInCroce(int pos1[][COLS], int * n)
{
    int x;

  x = random(2);
  if(x == 0){
   * n = 2;
   pos1[0][2] = 3;
  }else{
    * n = 4;
    pos1[1][1] = 3;
  }
}

void primaMossaDef(int pos1[][COLS], int * n, bool * bloccaMossaRandom)
{
  int n1;

  n1 = * n;
  if(pos1[1][1] == 5)
    primaMossaConSuaAlCentro(pos1, &n1);
  else if(pos1[0][0] == 5)
    primaMossaConSuaInAngolo(pos1, &n1);
  else if(pos1[0][1]==5)
    primaMossaConSuaInCroce(pos1, &n1);
  * bloccaMossaRandom = true;
  
   * n = n1;
}

void secondaMossaConSuaPrimaInCentro(int pos1[][COLS], int * n)
{
    * n = 2;
    pos1[0][2] = 3; 
}

void secondaMossaConSuaPrimaInAngolo(int pos1[][COLS], int * n)
{
    if(pos1[2][2]==5){
      * n = 1;
      pos1[0][1]=3;
    }else if(pos1[1][2]==5 || pos1[2][1]==5){
      * n = 8;
      pos1[2][2]=3;
    } 
}

void secondaMossaConSuaPrimaInCroce(int pos1[][COLS], int * n)
{
    if(pos1[1][1] == 3){
      if(pos1[1][0] == 5 || pos1[2][0] == 5 || pos1[2][1] == 5){
        * n = 0;
        pos1[0][0]=3;
      }
      else if(pos1[1][2]==5 || pos1[2][2]==5){
        * n = 2;
        pos1[0][2]=3;
      }
    }else if(pos1[0][2]==3){
      if(pos1[0][0]==5 || pos1[1][0]==5 || pos1[1][2]==5 || pos1[2][0]==5 || pos1[2][2]==5){
        * n = 4;
        pos1[1][1]=3;
      }
    }
}

void secondaMossaDef(int pos1[][COLS], int * n, int mossaDef, bool * bloccaMossaRandom)
{
  int n1;

  n1 = * n;
  if(mossaDef==1){
    if(pos1[1][1]==5 && pos1[0][0]==3 && pos1[2][2]==5)
      secondaMossaConSuaPrimaInCentro(pos1, &n1);
    else if(pos1[0][0]==5 && pos1[1][1]==3)
      secondaMossaConSuaPrimaInAngolo(pos1, &n1);
    else if(pos1[0][1]==5)
      secondaMossaConSuaPrimaInCroce(pos1, &n1);
    * bloccaMossaRandom = true;
  }
  * n = n1;
}

void controllavittoria(int pos1[][COLS], bool * vittoria)
{
    int stopp, i;
    stopp = 0;  
    for(i=0; i<3 && stopp == 0; i++)
      if(pos1[i][0]*pos1[i][1]*pos1[i][2]==27 || pos1[0][i]*pos1[1][i]*pos1[2][i]==27 || 
        pos1[0][0]*pos1[1][1]*pos1[2][2]==27 || pos1[0][2]*pos1[1][1]*pos1[2][0]==27){
          * vittoria = true;
          stopp = 1;
      }
}

void controllapareggio(int pos1[][COLS], bool * vittoria, bool * bloccaMovimento)
{
    if(pos1[0][0]+pos1[0][1]+pos1[0][2]+pos1[1][0]+pos1[1][1]+pos1[1][2]+pos1[2][0]+pos1[2][1]+pos1[2][2]==37 || 
      pos1[0][0]+pos1[0][1]+pos1[0][2]+pos1[1][0]+pos1[1][1]+pos1[1][2]+pos1[2][0]+pos1[2][1]+pos1[2][2]==35){
      * bloccaMovimento = true;
      * vittoria = true;
    }   
}

void Ruota(bool * ruotato1, int * angolo, int * n, int pos[][COLS], int pos1[][COLS], bool ruotato2, int turno)
{
  int stopp, h, i, j, val, val1;

  if(turno == 0){
    if(ruotato2 == false)
      val = 3;
    else if(ruotato2 == true)
      val = 5;
    val1 = 3;
  }else if(turno == 1){
    if(ruotato2 == false)
      val = 5;
    else if(ruotato2 == true)
      val = 3;
    val1 = 5;
  }
  
  if(* ruotato1 == false){
    stopp = 0;
    for(i = 0; i < 3 && stopp == 0; i++)
      for(j = 0; j < 3 && stopp == 0; j++)
        if(pos[i][j] == val){
          h = 3 * i + j;
          stopp = 1;
          * ruotato1 = true;
        }
    /*h è la posizione della mia pedina, in base ad essa ruoto il tris*/
    if(h == 0 || h == 1 || h == 4){
      ruota0(pos, pos1);
      * angolo = 0;
    }else if(h == 2 || h == 5){
      ruota90(pos, pos1);
      * angolo = 1;
    }else if(h == 7 || h == 8){
      ruota180(pos, pos1);
      * angolo = 2;
    }else if(h == 3 || h == 6){
      ruota270(pos, pos1);
      * angolo = 3;
    }
  }else{
    if(* angolo == 0)
      ruota0(pos, pos1);
    else if(* angolo == 1)
      ruota90(pos, pos1);
    else if(* angolo == 2)
      ruota180(pos, pos1);
    else if(* angolo == 3)
      ruota270(pos, pos1);
  }

  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++)
      if(pos1[i][j] == val1)
        * n = 3 * i + j;
}

void ruota0(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++)
      pos1[i][j] = pos[i][j];
}

void ruota90(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      pos1[i][j] = pos[j][abs(i-2)];
}

void ruota180(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      pos1[i][j] = pos[abs(i-2)][abs(j-2)];
}

void ruota270(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      pos1[i][j] = pos[abs(j-2)][i];
}

void riruota(int * angolo, int * n, int pos[][COLS], int pos1[][COLS])
{
  if(* angolo == 0)
    riruota0(pos, pos1);
  else if(* angolo == 1){
    riruota270(pos, pos1);
    if(* n == 0)
      * n = 2;
    else if(* n == 1)
      * n = 5;
    else if(* n == 2)
      * n = 8;
    else if(* n == 3)
      * n = 1;
    else if(* n == 4)
      * n = 4;
    else if(* n == 5)
      * n = 7;
    else if(* n == 6)
      * n = 0;
    else if(* n == 7)
      * n = 3;
    else if(* n == 8)
      * n = 6;   
  }
  else if(* angolo == 2){
    riruota180(pos, pos1);
    * n = abs((* n) - 8);
  }
  else if(* angolo == 3){
    riruota90(pos, pos1);
    if(* n == 0)
      * n = 6;
    else if(* n == 1)
      * n = 3;
    else if(* n == 2)
      * n = 0;
    else if(* n == 3)
      * n = 7;
    else if(* n == 4)
      * n = 4;
    else if(* n == 5)
      * n = 1;
    else if(* n == 6)
      * n = 8;
    else if(* n == 7)
      * n = 5;
    else if(* n == 8)
      * n = 2; 
  }
}

void riruota0(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++)
      pos[i][j] = pos1[i][j];
}

void riruota90(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      pos[i][j] = pos1[j][abs(i-2)];
}

void riruota180(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
       pos[i][j] = pos1[abs(i-2)][abs(j-2)];
}

void riruota270(int pos[][COLS], int pos1[][COLS])
{
  int i, j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      pos[i][j] = pos1[abs(j-2)][i];           
}

void chiudiMioTris(int pos1[][COLS], int * n, bool * verificaTris, bool * bloccaterzamossa, bool * bloccaMossaRandom, bool * vittoria)
{
    int i, j, stopp;
    
    stopp = 0;
    for(i=0; i<3 && stopp == 0; i++){ 
        /*Ho una riga da chiudere*/  
        if(pos1[i][0] * pos1[i][1] * pos1[i][2] == 9){
          for(j=0; j<3 && stopp == 0; j++)       
            if(pos1[i][j] == 1){
              * n = 3 * i + j;
              pos1[i][j]=3;
              stopp = 1;      
            }
        }
        /*Ho una colonna da chiudere*/
        else if (pos1[0][i] * pos1[1][i] * pos1[2][i] == 9){
          for(j=0; j<3 && stopp == 0; j++)     
            if(pos1[j][i] == 1){
              * n = 3 * j + i;
              pos1[j][i]=3;
              stopp = 1;
            }       
        }
        /*Ho una diagonale da chiudere*/
        else if (pos1[0][0] * pos1[1][1] * pos1[2][2] == 9 && pos1[i][i] == 1){
          * n = 3 * i + i;
          pos1[i][i]=3;
          stopp = 1; 
        }
        else if (pos1[0][2] * pos1[1][1] * pos1[2][0] == 9 && pos1[i][2-i] == 1){
          * n = 2 * i + 2;
          pos1[i][2-i]=3;       
          stopp = 1;                
        }   
    }
  if(stopp == 1){
    * verificaTris=true;
    * bloccaterzamossa=true;
    * bloccaMossaRandom=true;
    * vittoria=true;
  }     
}

void chiudiSuoTris(int pos1[][COLS], int * n, bool * bloccaterzamossa, bool * bloccaMossaRandom)
{  
    int i, j, stopp;
    
    stopp = 0;
    for(i=0; i<3 && stopp == 0; i++)
      { 
        /*Ho una riga da chiudere*/  
        if(pos1[i][0] * pos1[i][1] * pos1[i][2] == 25){
          for(j=0; j<3 && stopp == 0; j++)       
            if(pos1[i][j] == 1){
              * n = 3 * i + j;
              pos1[i][j]=3;
              stopp = 1;      
            }
        }
        /*Ho una colonna da chiudere*/
        else if (pos1[0][i] * pos1[1][i] * pos1[2][i] == 25){
          for(j=0; j<3 && stopp == 0; j++)     
            if(pos1[j][i] == 1){
              * n = 3 * j + i;
              pos1[j][i]=3;
              stopp = 1;
            }       
        }
        /*Ho una diagonale da chiudere*/
        else if (pos1[0][0] * pos1[1][1] * pos1[2][2] == 25 && pos1[i][i] == 1){
          * n = 3 * i + i;
          pos1[i][i]=3;
          stopp = 1; 
        }
        else if (pos1[0][2] * pos1[1][1] * pos1[2][0] == 25 && pos1[i][2-i] == 1){
          * n = 2 * i + 2;
          pos1[i][2-i]=3;       
          stopp = 1;                
        }   
      }
      if(stopp == 1){
        * bloccaterzamossa=true;
        * bloccaMossaRandom=true;
      }     
}
